/*
* OpenSpeedMonitor (OSM)
* Copyright 2014 iteratec GmbH
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
* 	http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/

package de.iteratec.osm.measurement.schedule

import de.iteratec.osm.measurement.environment.WebPageTestServer
import de.iteratec.osm.measurement.environment.wptserver.HttpRequestService
import de.iteratec.osm.result.WptStatus
import groovy.util.slurpersupport.GPathResult

class HttpRequestServiceMock extends HttpRequestService {
	public static final String testId = '140210_6W_1'
	public static
	final int[] statusCodes = [WptStatus.PENDING.getWptStatusCode(), WptStatus.IN_PROGRESS.getWptStatusCode(), WptStatus.IN_PROGRESS.getWptStatusCode(),
							   WptStatus.IN_PROGRESS.getWptStatusCode(), WptStatus.COMPLETED.getWptStatusCode()]
	public static final String redirectUserUrl = "http://dev.server01.wpt.iteratec.de/result/${testId}/"

	private int fetchResultCallCount = 0;

	// called from WptInstructionService.fetchResult()
	GPathResult getWptServerHttpGetResponse(WebPageTestServer wptserver, String path, Map query, String contentType, Map headers){
		fetchResultCallCount++
		int statusCode = fetchResultCallCount <= statusCodes.length ? statusCodes[fetchResultCallCount - 1] : 200
		String statusText
		switch(statusCode) {
			case WptStatus.PENDING.getWptStatusCode(): statusText = 'Test Pending'; break;
			case WptStatus.IN_PROGRESS.getWptStatusCode(): statusText = 'Test Running'; break;
			case WptStatus.COMPLETED.getWptStatusCode(): statusText = 'Ok'; break;
		}
		String xmlHeader = """\
<?xml version="1.0" encoding="UTF-8"?>
<response>
<requestId>${testId}</requestId>
<statusCode>${statusCode}</statusCode>
<statusText>${statusText}</statusText>
		"""

		String xmlFooter = '</response>'

		String resultData = """\
<data>
<testId>140214_5W_3</testId>
<summary>http://dev.server01.wpt.iteratec.de/result/140214_5W_3/</summary>
<testUrl>http://example.de/products</testUrl>
<location>iteratec-dev-netlab:Firefox</location>
<from>Firefox, Internet Explorer and Chrome - &lt;b&gt;Firefox&lt;/b&gt; - &lt;b&gt;Cable&lt;/b&gt;</from>
<connectivity>Cable</connectivity>
<bwDown>6000</bwDown>
<bwUp>512</bwUp>
<latency>50</latency>
<plr>0</plr>
<label>Chrome_Pad_ADS</label>
<completed>Fri, 14 Feb 2014 14:17:33 +0000</completed>
<testerDNS>80.69.32.3</testerDNS>
<runs>1</runs>
<successfulFVRuns>1</successfulFVRuns>
<successfulRVRuns>1</successfulRVRuns>
<average>
<firstView>
<URL>0</URL>
<loadTime>3780</loadTime>
<TTFB>1419</TTFB>
<bytesOut>26703</bytesOut>
<bytesOutDoc>26703</bytesOutDoc>
<bytesIn>639960</bytesIn>
<bytesInDoc>639960</bytesInDoc>
<connections>0</connections>
<requests>42</requests>
<requestsDoc>42</requestsDoc>
<responses_200>38</responses_200>
<responses_404>0</responses_404>
<responses_other>0</responses_other>
<result>0</result>
<render>2660</render>
<fullyLoaded>3780</fullyLoaded>
<cached>0</cached>
<docTime>3780</docTime>
<domTime>0</domTime>
<score_cache>50</score_cache>
<score_cdn>66</score_cdn>
<score_gzip>99</score_gzip>
<score_cookies>-1</score_cookies>
<score_keep-alive>100</score_keep-alive>
<score_minify>-1</score_minify>
<score_combine>50</score_combine>
<score_compress>91</score_compress>
<score_etags>-1</score_etags>
<gzip_total>239196</gzip_total>
<gzip_savings>1550</gzip_savings>
<minify_total>0</minify_total>
<minify_savings>0</minify_savings>
<image_total>225904</image_total>
<image_savings>19446</image_savings>
<optimization_checked>1</optimization_checked>
<aft>0</aft>
<domElements>2189</domElements>
<pageSpeedVersion>2</pageSpeedVersion>
<title>0</title>
<titleTime>1743</titleTime>
<loadEventStart>3780</loadEventStart>
<loadEventEnd>3928</loadEventEnd>
<domContentLoadedEventStart>3109</domContentLoadedEventStart>
<domContentLoadedEventEnd>3382</domContentLoadedEventEnd>
<lastVisualChange>3589</lastVisualChange>
<browser_name>0</browser_name>
<browser_version>26</browser_version>
<server_count>1</server_count>
<server_rtt>62</server_rtt>
<base_page_cdn>0</base_page_cdn>
<adult_site>0</adult_site>
<fixed_viewport>0</fixed_viewport>
<score_progressive_jpeg>0</score_progressive_jpeg>
<firstPaint>0</firstPaint>
<docCPUms>3463</docCPUms>
<fullyLoadedCPUms>3510</fullyLoadedCPUms>
<docCPUpct>23</docCPUpct>
<fullyLoadedCPUpct>23</fullyLoadedCPUpct>
<date>1392387452</date>
<SpeedIndex>2922</SpeedIndex>
<visualComplete>3600</visualComplete>
<effectiveBps>271054</effectiveBps>
<effectiveBpsDoc>271054</effectiveBpsDoc>
<avgRun>1</avgRun>
</firstView>
<repeatView>
<URL>0</URL>
<loadTime>1938</loadTime>
<TTFB>1227</TTFB>
<bytesOut>11787</bytesOut>
<bytesOutDoc>11787</bytesOutDoc>
<bytesIn>53937</bytesIn>
<bytesInDoc>53937</bytesInDoc>
<connections>0</connections>
<requests>15</requests>
<requestsDoc>15</requestsDoc>
<responses_200>4</responses_200>
<responses_404>0</responses_404>
<responses_other>0</responses_other>
<result>0</result>
<render>1802</render>
<fullyLoaded>1938</fullyLoaded>
<cached>1</cached>
<docTime>1938</docTime>
<domTime>0</domTime>
<score_cache>8</score_cache>
<score_cdn>-1</score_cdn>
<score_gzip>100</score_gzip>
<score_cookies>-1</score_cookies>
<score_keep-alive>100</score_keep-alive>
<score_minify>-1</score_minify>
<score_combine>100</score_combine>
<score_compress>-1</score_compress>
<score_etags>-1</score_etags>
<gzip_total>27302</gzip_total>
<gzip_savings>0</gzip_savings>
<minify_total>0</minify_total>
<minify_savings>0</minify_savings>
<image_total>0</image_total>
<image_savings>0</image_savings>
<optimization_checked>1</optimization_checked>
<aft>0</aft>
<domElements>2190</domElements>
<pageSpeedVersion>2</pageSpeedVersion>
<title>0</title>
<titleTime>1617</titleTime>
<loadEventStart>1939</loadEventStart>
<loadEventEnd>1940</loadEventEnd>
<domContentLoadedEventStart>1618</domContentLoadedEventStart>
<domContentLoadedEventEnd>1618</domContentLoadedEventEnd>
<lastVisualChange>1834</lastVisualChange>
<browser_name>0</browser_name>
<browser_version>26</browser_version>
<server_count>1</server_count>
<server_rtt>61</server_rtt>
<base_page_cdn>0</base_page_cdn>
<adult_site>0</adult_site>
<fixed_viewport>0</fixed_viewport>
<score_progressive_jpeg>-1</score_progressive_jpeg>
<firstPaint>0</firstPaint>
<docCPUms>1404</docCPUms>
<fullyLoadedCPUms>1420</fullyLoadedCPUms>
<docCPUpct>18</docCPUpct>
<fullyLoadedCPUpct>18</fullyLoadedCPUpct>
<date>1392387463</date>
<SpeedIndex>1800</SpeedIndex>
<visualComplete>1800</visualComplete>
<effectiveBps>75860</effectiveBps>
<effectiveBpsDoc>75860</effectiveBpsDoc>
<avgRun>1</avgRun>
</repeatView>
</average>
<standardDeviation>
<firstView>
<URL>0</URL>
<loadTime>0</loadTime>
<TTFB>0</TTFB>
<bytesOut>0</bytesOut>
<bytesOutDoc>0</bytesOutDoc>
<bytesIn>0</bytesIn>
<bytesInDoc>0</bytesInDoc>
<connections>0</connections>
<requests>0</requests>
<requestsDoc>0</requestsDoc>
<responses_200>0</responses_200>
<responses_404>0</responses_404>
<responses_other>0</responses_other>
<result>0</result>
<render>0</render>
<fullyLoaded>0</fullyLoaded>
<cached>0</cached>
<docTime>0</docTime>
<domTime>0</domTime>
<score_cache>0</score_cache>
<score_cdn>0</score_cdn>
<score_gzip>0</score_gzip>
<score_cookies>0</score_cookies>
<score_keep-alive>0</score_keep-alive>
<score_minify>0</score_minify>
<score_combine>0</score_combine>
<score_compress>0</score_compress>
<score_etags>0</score_etags>
<gzip_total>0</gzip_total>
<gzip_savings>0</gzip_savings>
<minify_total>0</minify_total>
<minify_savings>0</minify_savings>
<image_total>0</image_total>
<image_savings>0</image_savings>
<optimization_checked>0</optimization_checked>
<aft>0</aft>
<domElements>0</domElements>
<pageSpeedVersion>0</pageSpeedVersion>
<title>0</title>
<titleTime>0</titleTime>
<loadEventStart>0</loadEventStart>
<loadEventEnd>0</loadEventEnd>
<domContentLoadedEventStart>0</domContentLoadedEventStart>
<domContentLoadedEventEnd>0</domContentLoadedEventEnd>
<lastVisualChange>0</lastVisualChange>
<browser_name>0</browser_name>
<browser_version>0</browser_version>
<server_count>0</server_count>
<server_rtt>0</server_rtt>
<base_page_cdn>0</base_page_cdn>
<adult_site>0</adult_site>
<fixed_viewport>0</fixed_viewport>
<score_progressive_jpeg>0</score_progressive_jpeg>
<firstPaint>0</firstPaint>
<docCPUms>0</docCPUms>
<fullyLoadedCPUms>0</fullyLoadedCPUms>
<docCPUpct>0</docCPUpct>
<fullyLoadedCPUpct>0</fullyLoadedCPUpct>
<date>0</date>
<SpeedIndex>0</SpeedIndex>
<visualComplete>0</visualComplete>
<effectiveBps>0</effectiveBps>
<effectiveBpsDoc>0</effectiveBpsDoc>
<avgRun></avgRun>
</firstView>
<repeatView>
<URL>0</URL>
<loadTime>0</loadTime>
<TTFB>0</TTFB>
<bytesOut>0</bytesOut>
<bytesOutDoc>0</bytesOutDoc>
<bytesIn>0</bytesIn>
<bytesInDoc>0</bytesInDoc>
<connections>0</connections>
<requests>0</requests>
<requestsDoc>0</requestsDoc>
<responses_200>0</responses_200>
<responses_404>0</responses_404>
<responses_other>0</responses_other>
<result>0</result>
<render>0</render>
<fullyLoaded>0</fullyLoaded>
<cached>0</cached>
<docTime>0</docTime>
<domTime>0</domTime>
<score_cache>0</score_cache>
<score_cdn>0</score_cdn>
<score_gzip>0</score_gzip>
<score_cookies>0</score_cookies>
<score_keep-alive>0</score_keep-alive>
<score_minify>0</score_minify>
<score_combine>0</score_combine>
<score_compress>0</score_compress>
<score_etags>0</score_etags>
<gzip_total>0</gzip_total>
<gzip_savings>0</gzip_savings>
<minify_total>0</minify_total>
<minify_savings>0</minify_savings>
<image_total>0</image_total>
<image_savings>0</image_savings>
<optimization_checked>0</optimization_checked>
<aft>0</aft>
<domElements>0</domElements>
<pageSpeedVersion>0</pageSpeedVersion>
<title>0</title>
<titleTime>0</titleTime>
<loadEventStart>0</loadEventStart>
<loadEventEnd>0</loadEventEnd>
<domContentLoadedEventStart>0</domContentLoadedEventStart>
<domContentLoadedEventEnd>0</domContentLoadedEventEnd>
<lastVisualChange>0</lastVisualChange>
<browser_name>0</browser_name>
<browser_version>0</browser_version>
<server_count>0</server_count>
<server_rtt>0</server_rtt>
<base_page_cdn>0</base_page_cdn>
<adult_site>0</adult_site>
<fixed_viewport>0</fixed_viewport>
<score_progressive_jpeg>0</score_progressive_jpeg>
<firstPaint>0</firstPaint>
<docCPUms>0</docCPUms>
<fullyLoadedCPUms>0</fullyLoadedCPUms>
<docCPUpct>0</docCPUpct>
<fullyLoadedCPUpct>0</fullyLoadedCPUpct>
<date>0</date>
<SpeedIndex>0</SpeedIndex>
<visualComplete>0</visualComplete>
<effectiveBps>0</effectiveBps>
<effectiveBpsDoc>0</effectiveBpsDoc>
<avgRun></avgRun>
</repeatView>
</standardDeviation>
<median>
<firstView>
<run>1</run>
<URL>http://example.de/products</URL>
<loadTime>3780</loadTime>
<TTFB>1419</TTFB>
<bytesOut>26703</bytesOut>
<bytesOutDoc>26703</bytesOutDoc>
<bytesIn>639960</bytesIn>
<bytesInDoc>639960</bytesInDoc>
<connections>0</connections>
<requests>42</requests>
<requestsDoc>42</requestsDoc>
<responses_200>38</responses_200>
<responses_404>0</responses_404>
<responses_other>0</responses_other>
<result>0</result>
<render>2660</render>
<fullyLoaded>3780</fullyLoaded>
<cached>0</cached>
<docTime>3780</docTime>
<domTime>0</domTime>
<score_cache>50</score_cache>
<score_cdn>66</score_cdn>
<score_gzip>99</score_gzip>
<score_cookies>-1</score_cookies>
<score_keep-alive>100</score_keep-alive>
<score_minify>-1</score_minify>
<score_combine>50</score_combine>
<score_compress>91</score_compress>
<score_etags>-1</score_etags>
<gzip_total>239196</gzip_total>
<gzip_savings>1550</gzip_savings>
<minify_total>0</minify_total>
<minify_savings>0</minify_savings>
<image_total>225904</image_total>
<image_savings>19446</image_savings>
<optimization_checked>1</optimization_checked>
<aft>0</aft>
<domElements>2189</domElements>
<pageSpeedVersion>1.9</pageSpeedVersion>
<title>title of example.de</title>
<titleTime>1743</titleTime>
<loadEventStart>3780</loadEventStart>
<loadEventEnd>3928</loadEventEnd>
<domContentLoadedEventStart>3109</domContentLoadedEventStart>
<domContentLoadedEventEnd>3382</domContentLoadedEventEnd>
<lastVisualChange>3589</lastVisualChange>
<browser_name>Firefox</browser_name>
<browser_version>26.0.0.5087</browser_version>
<server_count>1</server_count>
<server_rtt>62</server_rtt>
<base_page_cdn></base_page_cdn>
<adult_site>0</adult_site>
<fixed_viewport>0</fixed_viewport>
<score_progressive_jpeg>0</score_progressive_jpeg>
<firstPaint>0</firstPaint>
<docCPUms>3463.222</docCPUms>
<fullyLoadedCPUms>3510.023</fullyLoadedCPUms>
<docCPUpct>23</docCPUpct>
<fullyLoadedCPUpct>23</fullyLoadedCPUpct>
<date>1392387452</date>
<SpeedIndex>2922</SpeedIndex>
<visualComplete>3600</visualComplete>
<effectiveBps>271054</effectiveBps>
<effectiveBpsDoc>271054</effectiveBpsDoc>
</firstView>
<repeatView>
<run>1</run>
<URL>http://example.de/products</URL>
<loadTime>1938</loadTime>
<TTFB>1227</TTFB>
<bytesOut>11787</bytesOut>
<bytesOutDoc>11787</bytesOutDoc>
<bytesIn>53937</bytesIn>
<bytesInDoc>53937</bytesInDoc>
<connections>0</connections>
<requests>15</requests>
<requestsDoc>15</requestsDoc>
<responses_200>4</responses_200>
<responses_404>0</responses_404>
<responses_other>0</responses_other>
<result>0</result>
<render>1802</render>
<fullyLoaded>1938</fullyLoaded>
<cached>1</cached>
<docTime>1938</docTime>
<domTime>0</domTime>
<score_cache>8</score_cache>
<score_cdn>-1</score_cdn>
<score_gzip>100</score_gzip>
<score_cookies>-1</score_cookies>
<score_keep-alive>100</score_keep-alive>
<score_minify>-1</score_minify>
<score_combine>100</score_combine>
<score_compress>-1</score_compress>
<score_etags>-1</score_etags>
<gzip_total>27302</gzip_total>
<gzip_savings>0</gzip_savings>
<minify_total>0</minify_total>
<minify_savings>0</minify_savings>
<image_total>0</image_total>
<image_savings>0</image_savings>
<optimization_checked>1</optimization_checked>
<aft>0</aft>
<domElements>2190</domElements>
<pageSpeedVersion>1.9</pageSpeedVersion>
<title>title of example.de</title>
<titleTime>1617</titleTime>
<loadEventStart>1939</loadEventStart>
<loadEventEnd>1940</loadEventEnd>
<domContentLoadedEventStart>1618</domContentLoadedEventStart>
<domContentLoadedEventEnd>1618</domContentLoadedEventEnd>
<lastVisualChange>1834</lastVisualChange>
<browser_name>Firefox</browser_name>
<browser_version>26.0.0.5087</browser_version>
<server_count>1</server_count>
<server_rtt>61</server_rtt>
<base_page_cdn></base_page_cdn>
<adult_site>0</adult_site>
<fixed_viewport>0</fixed_viewport>
<score_progressive_jpeg>-1</score_progressive_jpeg>
<firstPaint>0</firstPaint>
<docCPUms>1404.009</docCPUms>
<fullyLoadedCPUms>1419.609</fullyLoadedCPUms>
<docCPUpct>18</docCPUpct>
<fullyLoadedCPUpct>18</fullyLoadedCPUpct>
<date>1392387463</date>
<SpeedIndex>1800</SpeedIndex>
<visualComplete>1800</visualComplete>
<effectiveBps>75860</effectiveBps>
<effectiveBpsDoc>75860</effectiveBpsDoc>
</repeatView>
</median>
<run>
<id>1</id>
<firstView>
<results>
<URL>http://example.de/products</URL>
<loadTime>3780</loadTime>
<TTFB>1419</TTFB>
<bytesOut>26703</bytesOut>
<bytesOutDoc>26703</bytesOutDoc>
<bytesIn>639960</bytesIn>
<bytesInDoc>639960</bytesInDoc>
<connections>0</connections>
<requests>42</requests>
<requestsDoc>42</requestsDoc>
<responses_200>38</responses_200>
<responses_404>0</responses_404>
<responses_other>0</responses_other>
<result>0</result>
<render>2660</render>
<fullyLoaded>3780</fullyLoaded>
<cached>0</cached>
<docTime>3780</docTime>
<domTime>0</domTime>
<score_cache>50</score_cache>
<score_cdn>66</score_cdn>
<score_gzip>99</score_gzip>
<score_cookies>-1</score_cookies>
<score_keep-alive>100</score_keep-alive>
<score_minify>-1</score_minify>
<score_combine>50</score_combine>
<score_compress>91</score_compress>
<score_etags>-1</score_etags>
<gzip_total>239196</gzip_total>
<gzip_savings>1550</gzip_savings>
<minify_total>0</minify_total>
<minify_savings>0</minify_savings>
<image_total>225904</image_total>
<image_savings>19446</image_savings>
<optimization_checked>1</optimization_checked>
<aft>0</aft>
<domElements>2189</domElements>
<pageSpeedVersion>1.9</pageSpeedVersion>
<title>title of example.de</title>
<titleTime>1743</titleTime>
<loadEventStart>3780</loadEventStart>
<loadEventEnd>3928</loadEventEnd>
<domContentLoadedEventStart>3109</domContentLoadedEventStart>
<domContentLoadedEventEnd>3382</domContentLoadedEventEnd>
<lastVisualChange>3589</lastVisualChange>
<browser_name>Firefox</browser_name>
<browser_version>26.0.0.5087</browser_version>
<server_count>1</server_count>
<server_rtt>62</server_rtt>
<base_page_cdn></base_page_cdn>
<adult_site>0</adult_site>
<fixed_viewport>0</fixed_viewport>
<score_progressive_jpeg>0</score_progressive_jpeg>
<firstPaint>0</firstPaint>
<docCPUms>3463.222</docCPUms>
<fullyLoadedCPUms>3510.023</fullyLoadedCPUms>
<docCPUpct>23</docCPUpct>
<fullyLoadedCPUpct>23</fullyLoadedCPUpct>
<date>1392387452</date>
<SpeedIndex>2922</SpeedIndex>
<visualComplete>3600</visualComplete>
<effectiveBps>271054</effectiveBps>
<effectiveBpsDoc>271054</effectiveBpsDoc>
</results>
<pages>
<details>http://dev.server01.wpt.iteratec.de/result/140214_5W_3/1/details/</details>
<checklist>http://dev.server01.wpt.iteratec.de/result/140214_5W_3/1/performance_optimization/</checklist>
<breakdown>http://dev.server01.wpt.iteratec.de/result/140214_5W_3/1/breakdown/</breakdown>
<domains>http://dev.server01.wpt.iteratec.de/result/140214_5W_3/1/domains/</domains>
<screenShot>http://dev.server01.wpt.iteratec.de/result/140214_5W_3/1/screen_shot/</screenShot>
</pages>
<thumbnails>
<waterfall>http://dev.server01.wpt.iteratec.de/result/140214_5W_3/1_waterfall_thumb.png</waterfall>
<checklist>http://dev.server01.wpt.iteratec.de/result/140214_5W_3/1_optimization_thumb.png</checklist>
<screenShot>http://dev.server01.wpt.iteratec.de/result/140214_5W_3/1_screen_thumb.jpg</screenShot>
</thumbnails>
<images>
<waterfall>http://dev.server01.wpt.iteratec.de/results/14/02/14/5W/3/1_waterfall.png</waterfall>
<connectionView>http://dev.server01.wpt.iteratec.de/results/14/02/14/5W/3/1_connection.png</connectionView>
<checklist>http://dev.server01.wpt.iteratec.de/results/14/02/14/5W/3/1_optimization.png</checklist>
<screenShot>http://dev.server01.wpt.iteratec.de/results/14/02/14/5W/3/1_screen.jpg</screenShot>
</images>
<rawData><headers>http://dev.server01.wpt.iteratec.de/results/14/02/14/5W/3/1_report.txt</headers>
<bodies>http://dev.server01.wpt.iteratec.de/results/14/02/14/5W/3/1_bodies.zip</bodies>
<pageData>http://dev.server01.wpt.iteratec.de/results/14/02/14/5W/3/1_IEWPG.txt</pageData>
<requestsData>http://dev.server01.wpt.iteratec.de/results/14/02/14/5W/3/1_IEWTR.txt</requestsData>
<utilization>http://dev.server01.wpt.iteratec.de/results/14/02/14/5W/3/1_progress.csv</utilization>
</rawData>
<videoFrames>
<frame>
<time>0</time>
<image>http://dev.server01.wpt.iteratec.de/results/14/02/14/5W/3/video_1/frame_0000.jpg</image>
<VisuallyComplete>0</VisuallyComplete>
</frame>
<frame>
<time>2700</time>
<image>http://dev.server01.wpt.iteratec.de/results/14/02/14/5W/3/video_1/frame_0027.jpg</image>
<VisuallyComplete>48</VisuallyComplete>
</frame>
<frame>
<time>3000</time>
<image>http://dev.server01.wpt.iteratec.de/results/14/02/14/5W/3/video_1/frame_0030.jpg</image>
<VisuallyComplete>89</VisuallyComplete>
</frame>
<frame>
<time>3600</time>
<image>http://dev.server01.wpt.iteratec.de/results/14/02/14/5W/3/video_1/frame_0036.jpg</image>
<VisuallyComplete>100</VisuallyComplete>
</frame>
</videoFrames>
</firstView>
<repeatView>
<results>
<URL>http://example.de/products</URL>
<loadTime>1938</loadTime>
<TTFB>1227</TTFB>
<bytesOut>11787</bytesOut>
<bytesOutDoc>11787</bytesOutDoc>
<bytesIn>53937</bytesIn>
<bytesInDoc>53937</bytesInDoc>
<connections>0</connections>
<requests>15</requests>
<requestsDoc>15</requestsDoc>
<responses_200>4</responses_200>
<responses_404>0</responses_404>
<responses_other>0</responses_other>
<result>0</result>
<render>1802</render>
<fullyLoaded>1938</fullyLoaded>
<cached>1</cached>
<docTime>1938</docTime>
<domTime>0</domTime>
<score_cache>8</score_cache>
<score_cdn>-1</score_cdn>
<score_gzip>100</score_gzip>
<score_cookies>-1</score_cookies>
<score_keep-alive>100</score_keep-alive>
<score_minify>-1</score_minify>
<score_combine>100</score_combine>
<score_compress>-1</score_compress>
<score_etags>-1</score_etags>
<gzip_total>27302</gzip_total>
<gzip_savings>0</gzip_savings>
<minify_total>0</minify_total>
<minify_savings>0</minify_savings>
<image_total>0</image_total>
<image_savings>0</image_savings>
<optimization_checked>1</optimization_checked>
<aft>0</aft>
<domElements>2190</domElements>
<pageSpeedVersion>1.9</pageSpeedVersion>
<title>title of example.de</title>
<titleTime>1617</titleTime>
<loadEventStart>1939</loadEventStart>
<loadEventEnd>1940</loadEventEnd>
<domContentLoadedEventStart>1618</domContentLoadedEventStart>
<domContentLoadedEventEnd>1618</domContentLoadedEventEnd>
<lastVisualChange>1834</lastVisualChange>
<browser_name>Firefox</browser_name>
<browser_version>26.0.0.5087</browser_version>
<server_count>1</server_count>
<server_rtt>61</server_rtt>
<base_page_cdn></base_page_cdn>
<adult_site>0</adult_site>
<fixed_viewport>0</fixed_viewport>
<score_progressive_jpeg>-1</score_progressive_jpeg>
<firstPaint>0</firstPaint>
<docCPUms>1404.009</docCPUms>
<fullyLoadedCPUms>1419.609</fullyLoadedCPUms>
<docCPUpct>18</docCPUpct>
<fullyLoadedCPUpct>18</fullyLoadedCPUpct>
<date>1392387463</date>
<SpeedIndex>1800</SpeedIndex>
<visualComplete>1800</visualComplete>
<effectiveBps>75860</effectiveBps>
<effectiveBpsDoc>75860</effectiveBpsDoc>
</results>
<pages>
<details>http://dev.server01.wpt.iteratec.de/result/140214_5W_3/1/details/cached/</details>
<checklist>http://dev.server01.wpt.iteratec.de/result/140214_5W_3/1/performance_optimization/cached/</checklist>
<report>http://dev.server01.wpt.iteratec.de/result/140214_5W_3/1/optimization_report/cached/</report>
<breakdown>http://dev.server01.wpt.iteratec.de/result/140214_5W_3/1/breakdown/</breakdown>
<domains>http://dev.server01.wpt.iteratec.de/result/140214_5W_3/1/domains/</domains>
<screenShot>http://dev.server01.wpt.iteratec.de/result/140214_5W_3/1/screen_shot/cached/</screenShot>
</pages>
<thumbnails>
<waterfall>http://dev.server01.wpt.iteratec.de/result/140214_5W_3/1_Cached_waterfall_thumb.png</waterfall>
<checklist>http://dev.server01.wpt.iteratec.de/result/140214_5W_3/1_Cached_optimization_thumb.png</checklist>
<screenShot>http://dev.server01.wpt.iteratec.de/result/140214_5W_3/1_Cached_screen_thumb.jpg</screenShot>
</thumbnails>
<images>
<waterfall>http://dev.server01.wpt.iteratec.de/results/14/02/14/5W/3/1_Cached_waterfall.png</waterfall>
<connectionView>http://dev.server01.wpt.iteratec.de/results/14/02/14/5W/3/1_Cached_connection.png</connectionView>
<checklist>http://dev.server01.wpt.iteratec.de/results/14/02/14/5W/3/1_Cached_optimization.png</checklist>
<screenShot>http://dev.server01.wpt.iteratec.de/results/14/02/14/5W/3/1_Cached_screen.jpg</screenShot>
</images>
<rawData>
<headers>http://dev.server01.wpt.iteratec.de/results/14/02/14/5W/3/1_Cached_report.txt</headers>
<bodies>http://dev.server01.wpt.iteratec.de/results/14/02/14/5W/3/1_Cached_bodies.zip</bodies>
<pageData>http://dev.server01.wpt.iteratec.de/results/14/02/14/5W/3/1_Cached_IEWPG.txt</pageData>
<requestsData>http://dev.server01.wpt.iteratec.de/results/14/02/14/5W/3/1_Cached_IEWTR.txt</requestsData>
<utilization>http://dev.server01.wpt.iteratec.de/results/14/02/14/5W/3/1_Cached_progress.csv</utilization>
</rawData>
<videoFrames>
<frame>
<time>0</time>
<image>http://dev.server01.wpt.iteratec.de/results/14/02/14/5W/3/video_1_cached/frame_0000.jpg</image>
<VisuallyComplete>0</VisuallyComplete>
</frame>
<frame>
<time>1800</time>
<image>http://dev.server01.wpt.iteratec.de/results/14/02/14/5W/3/video_1_cached/frame_0018.jpg</image>
<VisuallyComplete>100</VisuallyComplete>
</frame>
</videoFrames>
</repeatView>
</run>
</data>
		"""
		return parseXml([ data: [ text: xmlHeader + (statusCode == WptStatus.COMPLETED.getWptStatusCode() ? resultData : '') + xmlFooter ]])
	}
    GPathResult parseXml(Object xmlResponse) {
        assert xmlResponse != null
        def myresponse = new XmlSlurper().parseText(xmlResponse.data.text)
        assert myresponse != null
        return myresponse
    }
}
